#include "audioc.cmdline.h"
#include "para.h"

struct gengetopt_args_info conf;
char *tmpfifo;
enum {E_SYNTAX, E_READ, E_WRITE, E_SOCKET, E_INIT_SOCK_ADDR, E_CONNECT, E_CREDENTIALS};


void clog(int ll, char* fmt,...) /* no logging */
{
}

static char *configfile_exists(void)
{
	static char *config_file;
        struct stat statbuf;


	if (!config_file) {
		char *home = para_homedir();
		config_file = make_message("%s/.paraslash/audioc.conf", home);
		free(home);
	}
        if (!stat(config_file, &statbuf))
		return config_file;
	return NULL;
}

int main(int argc, char *argv[])
{
	struct sockaddr_un unix_addr;
	int ret = -E_SYNTAX, fd;
	char *cf, *socket_name, *randname = para_tmpname(), *tmpsocket_name = NULL,
		*buf = para_malloc(STRINGSIZE), *hn = para_hostname(), *args,
		*home = para_homedir();

	
	if (cmdline_parser(argc, argv, &conf))
		goto out;
	cf = configfile_exists();
	if (cf) {
		if (cmdline_parser_configfile(cf, &conf, 0, 0, 0)) {
			fprintf(stderr, "parse error in config file\n");
			exit(EXIT_FAILURE);
		}
	}
	args = conf.inputs_num?
		concat_args(conf.inputs_num, conf.inputs) :
		para_strdup("stat");
	
	if (conf.socket_given)
		socket_name = para_strdup(conf.socket_arg);
	else
		socket_name = make_message(
			"/var/paraslash/audiod_socket.%s", hn);
	if (conf.tmpdir_given)
		tmpsocket_name = make_message("%s/audioc.sock.%s.%s",
			conf.tmpdir_arg, hn, randname);
	else
		tmpsocket_name = make_message("%s/.paraslash/audioc_sock.%s.%s",
			home, hn, randname);

	ret = -E_SOCKET;
	fd = create_pf_socket(tmpsocket_name, &unix_addr, S_IRUSR | S_IWUSR);
	unlink(tmpsocket_name);
	if (fd < 0)
		goto out;
	ret = -E_INIT_SOCK_ADDR;
	if (init_unix_addr(&unix_addr, socket_name) < 0)
		goto out;
	ret = - E_CONNECT;
	if (connect(fd, (struct sockaddr *)&unix_addr, UNIX_PATH_MAX) < 0)
		goto out;
	ret = -E_CREDENTIALS;
	if (send_cred_buffer(fd, args) < 0)
		goto out;
	for (;;) {
		ssize_t len = recv_buffer(fd, buf, STRINGSIZE);
		if (len <= 0) {
			ret = len < 0? -E_READ : 0;
			goto out;
		}
		ret = write(STDOUT_FILENO, buf, len);
		if (len != ret) {
			ret = -E_WRITE;
			goto out;
		}
	}
out:
	return ret < 0? -ret : EXIT_SUCCESS;
}
